#include "rules_int.h"
#include "ai_int.h"
#include "pdn.h"
#include "netgame.h"
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

/* TYPES" */
typedef struct{
	int type;
	char *cmd_string;
	Move *move;
} Command;
/* types of command */
#define NONE 0
#define HELP 1
#define OPEN 2
#define SAVE 3
#define QUIT 4
#define MOVE 5

/* FUNCTION PROTOTYPES */

int play(Position *position, const Player me, const int opponent);
void print_board(const Position *position);
void print_help();

int i_am_on_move(const Player on_move, const Player me);
Command input_command(Position *position);
Command parse(char *cmd, Position *position);
int execute_command(const Command command, Position *position);

void send_command(const Command command, const int opponent);
Command receive_command(const int opponent, Position *position);

void free_command(Command command);


/* GLOBAL VARIABLES */

/****/

int main(int argc, char *argv[])
{
	int port= 55231; 
	char host[64];
//	char str[64];
	Position position;
	Player me= WHITE;
	Move move;
	int difficulty = 2;
	int mode = 3; /* server = 1, client = 2, local = 3 */
	int opponent;
	int opt;
	/* parse command line options */
	while((opt = getopt(argc, argv, "p:sc:d:")) != -1){
		switch(opt) {
			case 'p':
				port = atoi(optarg);
			case 's':
				me= BLACK;
				mode = 1;
				break;
			case 'c':
				mode = 2;
				strcpy(host, optarg);
				break;
			case 'd':
				mode = 3;
				difficulty = atoi(optarg);
				break;
			case 'h':
				break;
		}
	}
	switch(mode) {
		case 1:
			opponent = initialize_server(port);
			break;
		case 2:
			opponent = initialize_client(host, port);
			break;
		default:
			opponent = -difficulty;
			break;
	}
	
	initialize_board(&position);
	while(get_game_state(&position, 0, &move) == PROCEEDING){
		print_board(&position);
		if(!play(&position, me, opponent)) printf("error");
	}
	printf("Game over\n");
	
	return EXIT_SUCCESS;
} 

int play(Position *position, const Player me, const int opponent)
{
	Command command;
	command.type = NONE;
	command.cmd_string = NULL;
	command.move = NULL;
	
	if(i_am_on_move(position->onmove, me)){
		while(command.type != MOVE){ 
			execute_command(command, position);
			command= input_command(position);
		}
		execute_command(command, position);
		send_command(command, opponent);
		free_command(command);
	}else{
//		printf("DEBUG: receive command:\n");
		command= receive_command(opponent, position);
		execute_command(command, position);
		free_command(command);
	}
	return 1;	
}

void print_board(const Position *position)
{
	int i, j;
	char square[3];
	Sq_coords coords;
	
	printf("+--+--+--+--+--+--+--+--+--+--+\n");
	for(j=0; j<BOARDSIZE; j++){
		for(i=0; i<BOARDSIZE; i++){
			coords.x = i; coords.y = j;
			if(is_black_square(i,j)){
				sprintf(square,"%02d", coords2no(coords));
			}else{
				strcpy(square, "  ");
			}
					
			switch(position->placement[i][j]){
				case WHITE_PAWN: printf("|\033[01;33m%s\033[00m", square); break;
				case BLACK_PAWN: printf("|\033[01;31m%s\033[00m", square); break;
				case WHITE_KING: printf("|\033[01;33m%s\033[00m", "DD"); break;
				case BLACK_KING: printf("|\033[01;31m%s\033[00m", "DD"); break;
				default: printf("|%s", square);
			}
		}
		printf("|\n");
		printf("+--+--+--+--+--+--+--+--+--+--+\n");
	}
	printf("\n");
} 

void print_help()
{ 
	printf("Commands: \n");
	printf("?, h ... this help\n");
	printf("s <filename> ... save game\n");
	printf("o <filename> ... open game\n");
	printf("16-21 ... move\n");
	printf("q ... quit\n");
	printf("\n");
}

int i_am_on_move(const Player on_move, const Player me)
{ 
	if((on_move * me) > 0) return 1;
	else return 0;
}

Command input_command(Position *position)
{
	char *cmd;
	Command command;
	
	cmd = (char *)malloc(64*sizeof(char));
	printf("Your command: "); 
	fgets(cmd, 64, stdin);
	command= parse(cmd, position);
	return command;
}

Command parse(char *cmd, Position *position)
{
	Command command;
	Move move;
	Moves poss_moves;

	command.cmd_string= cmd;
	switch(cmd[0]){
	case '?':
	case 'h':
		command.type = HELP;
		command.move = NULL;
//		printf("h\n");
		break;
	case 'q':
		command.type = QUIT;
		command.move = NULL;
//		printf("q\n");
		break;
	case 'o':
		command.type = OPEN;
		command.move = NULL;
//		printf("o\n");
		break;
	case 's':
		command.type = SAVE;
		command.move = NULL;
//		printf("s\n");
		break;
	default:
//		printf("Parsing command type MOVE\n");
		command.type = MOVE;
		str2move(cmd, &move);
		poss_moves= get_all_moves(position);
		command.move= (Move *)malloc(sizeof(Move));
		if( illegal_move(*command.move= match_move(move, &poss_moves))){
			printf("Illegal move or bad command\n");
			free(command.move);
			command.type = NONE;
		}
//		printf("DEBUG: Start coordinates: %d, %d\n", command.move->start.x, command.move->start.y);
	}
	return command;
}

int execute_command(const Command command, Position *position)
{
	int ret;
		
	switch(command.type){
	case NONE:
		break;
	case HELP: 
		printf("You want help.\n");
		print_help(); 
		break;
	case OPEN: 
		command.cmd_string[strlen(command.cmd_string)-1] = '\0';
		open_game(command.cmd_string + 2, position);
		printf("Game loaded\n");
		print_board(position);
		break;
	case SAVE: 
		command.cmd_string[strlen(command.cmd_string)-1] = '\0';
		save_game(command.cmd_string + 2);
		printf("Game saved.\n");
		break;
	case QUIT:
		exit(0);
		break;
	case MOVE:
		insert_move(*command.move);
//		printf("do_move, onmove is: %d\n", position->onmove);
		do_move(position, *command.move);
		break;
	default:
		printf("command type is: %d\n", command.type);
		break;
	}
	return ret;
		   
}

void send_command(const Command command, const int opponent)
{
	
	if(opponent < 0){
		/* playing against computer, not using the net */
		/* no action */
	}else{
//		printf("Sending move to opponent... ");
		netwrite(opponent, command.cmd_string);
		netwrite(opponent, "$");
//		printf("Done. I sent %s\n", command.cmd_string);
	}
}

Command receive_command(const int opponent, Position *position)
{
	Move *move = (Move *)malloc(sizeof(Move));
	char *cmd = (char *)malloc(64 * sizeof(char));
	Command command;
	
	if(opponent < 0){
		/* playing against computer, not using the net */
		printf("Computer thinking...\n");
//		printf("choose_best_move:, opponent == %d\n", opponent);
		*move= choose_best_move(position, -opponent);
//		printf("move2str:\n");
		move2str(move, cmd);
		printf("Opponent's move: %s\n", cmd);
		command.type= MOVE;
		command.cmd_string= cmd;
		command.move= move;
	}else{
		printf("Waiting for opponent's move...\n");
		netread(opponent, cmd);
		printf("Opponent's move %s\n", cmd);
//		printf("Parsing cmd_string\n");
		command= parse(cmd, position);
	}
//	printf("Leaving receive_command\n");
	return command;
}
		
void free_command(Command command)
{
	if(command.cmd_string != NULL) free(command.cmd_string);
	if(command.move != NULL) free(command.move);
}
