/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * pdn.c                                                     *
 * part of checkers project, contains functions to keep      *
 * history of moves, to save and load games. uses standard   *
 * format PDN (Portable draughts notation)                   *
 * Miroslav Tynovsky 07/2004                                 *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <stdlib.h>
#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "rules_int.h"
#include "pdn.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define BUFFSZ 64

Moves history;

int save_game(const char *filename)
{
	history2pdn(filename);
	printf("Game saved\n");
	return 1;
}

int open_game(const char *filename, Position *position)
{
	pdn2history(filename);
	load_from_history(position);
	return 1;
}

void insert_move(Move move)
{
	if(history.allocated==0){
		history.allocated= 4;
		history.list= (Move *)malloc(history.allocated * sizeof(Move));
	}
	if(history.allocated==history.count){
		history.allocated*= 2;
		history.list= realloc(history.list, history.allocated * sizeof(Move));
	}
	history.list[history.count]= move;
	history.count++;
}

void history2pdn(const char *filename)
{
	int i;
	char str[64];
	int f;

	f= open(filename, O_WRONLY | O_CREAT | O_TRUNC);
	for(i=0; i<history.count; i++){
		if(i%2==0){ /*print move number*/
			sprintf(str, " %d. ", i/2+1); 
			write(f, str, strlen(str)); 
		}
		move2str(&history.list[i], str); 
		write(f, str, strlen(str)); /*print move*/
		if(i%10==9) write(f, "\n", strlen("\n"));
	}
	write(f, "\n", strlen("\n"));
	close(f);
}

void pdn2history(const char *filename)
{
	int i, j, k, start, dest;
	char ch;
	int middle[11];
	FILE *f;
	Move move;
	Moves poss_moves;
	Position pos;

	initialize_board(&pos);
	history.count= 0;
	if(history.allocated > 0){
		history.allocated= 0;
		free(history.list);
	}

	f= fopen(filename, "r");
	for(i=0; fscanf(f, "%d. ", &start)!=EOF; i++){ 
		//uvnitr for se prirazuje do start, jen aby se nekam priradilo. 
		//hodnota se hned zahodi, je to jen poradove cislo tahu
		fscanf(f, "%d", &start);
		for(j=0; ((ch= getc(f))!=' ') && (ch != EOF); j++)
			fscanf(f, "%d", middle+j);
		assert(j>0);
		assert(ch != EOF);
		dest= middle[j-1];
		middle[j-1]= -1;
		move.start= no2coords(start);
		move.destination= no2coords(dest);
		for(k=0; k<j; k++)
			move.middle[k]= no2coords(middle[k]);
		poss_moves= get_all_moves(&pos);
		move= match_move(move, &poss_moves);
		assert(move.start.x > -1);
		do_move(&pos, move);
		insert_move(move);
		
		if( fscanf(f, "%d", &start) == EOF ) break;
		for(j=0; ((ch= getc(f))!=' ') && (ch != EOF); j++)
			fscanf(f, "%d", middle+j);
		assert(j>0);
		assert(ch != EOF);
		dest= middle[j-1];
		middle[j]= -1;
		move.start= no2coords(start);
		move.destination= no2coords(dest);
		for(k=0; k<j; k++)
			move.middle[k]= no2coords(middle[k]);
		poss_moves= get_all_moves(&pos);
		move= match_move(move, &poss_moves);
		assert(move.start.x > -1);
		do_move(&pos, move);
		insert_move(move);
	}
	fclose(f);
}

void load_from_history(Position *position)
{
	int i;

	initialize_board(position);
	for(i=0; i<history.count; i++)
		do_move(position, history.list[i]);
}

int coords2no(Sq_coords coords)
{
	return (coords.x/2 + 1) + coords.y*5;
}

Sq_coords no2coords(int no)
{
	Sq_coords coords;
	coords.x= ((no - 1) % 5) * 2 + (((no - 1) % 10) / 5);
	coords.y= (no - 1) / 5;
	return coords;
}

void str2move(const char *str, Move *move)
{
	int j, k, start, dest, middle[11];
	char *ch, *token;
	char s[64];

	strcpy(s, str);

	while((ch= strchr(s, 'x'))) 
		*ch='-';
//	printf("str2move: going to parse: %s\n", s);
	start= atoi(strtok(s, "-"));
//	printf("Start is: %d\n", start);
	for(j=0; (token= strtok(NULL, "-")) != NULL; j++){
		middle[j]= atoi(token);
//		printf("Middle %d is: %d\n", j, middle[j]);
	}
	if(j<=0){
//		printf("str2move: this move has no destination\n");
		move->start.x= -1;
		return;
	}
	dest= middle[j-1];
	middle[j-1]= -1;
	move->start= no2coords(start);
	move->destination= no2coords(dest);
	for(k=0; k<j; k++)
		move->middle[k]= no2coords(middle[k]);
}

void move2str(const Move *move, char *str)
{
	int i;

	switch(move->move_type){
	case PAWN_STEP:
	case KING_STEP:
	case PROMOTION:
		sprintf(str, "%d-%d ", coords2no(move->start), coords2no(move->destination));
		break;
	case PAWN_JUMP:
	case KING_JUMP:
	case JUMP_PROMOTION:
		sprintf(str, "%dx", coords2no(move->start));
		for(i=0; i<move->jump_length-1; i++)
			sprintf(str, "%s%dx", str, coords2no(move->middle[i]));
		sprintf(str, "%s%d ", str, coords2no(move->destination));
		break;
	default: /* this should never happen */
		sprintf(str, "%d-%d (movetype %d)", 
		        coords2no(history.list[i].start), 
		        coords2no(history.list[i].destination),
		        history.list[i].move_type);
		break;
	}
}	
	
